@@ -3,6 +3,7 @@ use strict;
 use warnings;
 use Math::Prime::Util;
 use Math::Prime::Util::GMP;
+use Math::Prime::Util::PP;
 use Math::Primality;
 use Benchmark qw/:all/;
 my $count = shift || -2;
@@ -13,8 +14,8 @@ my($n, $exp) = (1000000,78498);
 cmpthese($count,{
   'MP'           =>sub { die unless $exp == Math::Primality::prime_count($n); },
   'MPU default'  =>sub { die unless $exp == Math::Prime::Util::prime_count($n); },
-  'MPU XS Sieve' =>sub { die unless $exp == Math::Prime::Util::_XS_prime_count($n); },
-  'MPU XS Lehmer'=>sub { die unless $exp == Math::Prime::Util::_XS_lehmer_pi($n); },
+  'MPU XS Sieve' =>sub { die unless $exp == Math::Prime::Util::_XS_segment_pi($n); },
+  #'MPU XS Lehmer'=>sub { die unless $exp == Math::Prime::Util::_XS_lehmer_pi($n); },
   'MPU PP Sieve' =>sub { die unless $exp == Math::Prime::Util::PP::_sieve_prime_count($n); },
   'MPU PP Lehmer'=>sub { die unless $exp == Math::Prime::Util::PP::_lehmer_pi($n); },
   'MPU GMP Trial'=>sub { die unless $exp == Math::Prime::Util::GMP::prime_count(2,$n); },
